@@ -19,13 +19,13 @@
 ;; 1) does NOT parse lone expressions of the form [ 1, 2] => syntax error
 ;; 2) does NOT support non-comma rows [ 1 2 ] => syntax error
 
-(define-module (lang matlab pgen)
+(define-module (nyacc lang matlab pgen)
   #:export (matlab-spec
 	    matlab-mach
 	    gen-matlab-lexer
 	    matlab-parser
 	    parse-m)
-  #:use-module (lang util)
+  #:use-module (nyacc lang util)
   #:use-module (nyacc lalr)
   #:use-module (nyacc lex)
   #:use-module (ice-9 pretty-print)
@@ -33,7 +33,7 @@
 
 (define matlab-spec
   (lalr-spec
-   (notice lang-crn)
+   (notice lang-crn-lic)
    (start mfile)
    (grammar
     
@@ -247,7 +247,7 @@
 
     (term (#\newline) (";") (","))
     (ident ('$ident ($$ `(ident ,$1))))
-    (number ('$fx ($$ `(fixed ,$1))) ('$fl ($$ `(float ,$1))))
+    (number ('$fixed ($$ `(fixed ,$1))) ('$float ($$ `(float ,$1))))
     (string ('$string ($$ `(string ,$1))))
     (lone-comment ('$lone-comm ($$ `(comm ,$1))))
     ;;(code-comment ('$code-comm ($$ `(comm ,$1))))
