@@ -17,7 +17,7 @@
 
 ;; idea: make changes and call it Modzillica
 
-(define-module (lang modelica pgen)
+(define-module (nyacc lang modelica pgen)
   #:export (modelica-spec
 	    modelica-mach
 	    modelica-parser
@@ -25,7 +25,7 @@
 	    parse-mo)
   #:use-module (nyacc lalr)
   #:use-module (nyacc lex)
-  #:use-module (lang util)
+  #:use-module (nyacc lang util)
   #:use-module (ice-9 pretty-print)
   )
 
@@ -35,7 +35,7 @@
 
 (define modelica-spec
   (lalr-spec
-   (notice lang-crn)
+   (notice lang-crn-lic)
    (start stored-definition)
    (grammar
     
@@ -661,8 +661,8 @@
 
     ;; end of grammar
     (unsigned-number
-     ('$fx ($$ `(unsigned-number ,$1)))
-     ('$fl ($$ `(unsigned-number ,$1))))
+     ('$fixed ($$ `(unsigned-number ,$1)))
+     ('$float ($$ `(unsigned-number ,$1))))
     (ident ('$ident ($$ `(ident ,$1))))
     (string ('$string ($$ `(string ,$1))))
     )))
