@@ -16,18 +16,18 @@
 ;;; License along with this library; if not, write to the Free Software
 ;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+;; I need to find way to preserve srconf, rrconf after hashify.
+;; compact needs to deal with it ...
+
 (define-module (nyacc lalr)
   #:export-syntax (lalr-spec)
   #:export (*nyacc-version*
 	    make-lalr-machine
 	    compact-machine hashify-machine
 	    lalr-match-table
-	    make-lalr-parser
 	    pp-lalr-grammar pp-lalr-machine
 	    write-lalr-tables
 	    write-lalr-actions
-	    ;;write-lalr-parser
-	    wrap-action
 	    pp-rule find-terminal	; used by (nyacc export)
 	    with-gram new-non-kernels
 	    )
@@ -37,9 +37,10 @@
   #:use-module ((srfi srfi-9) #:select (define-record-type))
   #:use-module ((srfi srfi-43) #:select (vector-map vector-for-each))
   #:use-module (nyacc util)
+  #:use-module ((nyacc parse) #:select (wrap-action))
   )
 
-(define *nyacc-version* "0.62.0")
+(define *nyacc-version* "0.64.0")
 
 ;; @item proxy-? sym rhs
 ;; @example
@@ -115,16 +116,19 @@
 	    (parse-rhs
 	     (lambda (x)
 	       ;; The following is syntax-case because we use a fender.
-	       (syntax-case x (quote $$ $$/ref $$-ref $with $? $* $+)
+	       (syntax-case x (quote $$ $$/ref $$-ref $prec $with $? $* $+)
 		 ;; action specifications
 		 ((_ ($$ <guts> ...) <e2> ...)
 		  #'(cons '(action #f #f <guts> ...) (parse-rhs <e2> ...)))
 		 ((_ ($$-ref <ref>) <e2> ...)
-		  #'(cons '(action #f '<ref> #f) (parse-rhs <e2> ...)))
+		  ;;#'(cons '(action #f <ref> #f) (parse-rhs <e2> ...)))
+		  #'(cons `(action #f ,<ref> . #f) (parse-rhs <e2> ...)))
 		 ((_ ($$/ref <ref> <guts> ...) <e2> ...)
-		  #'(cons '(action #f <ref> <guts> ...) (parse-rhs <e2> ...)))
+		  #'(cons `(action #f ,<ref> <guts> ...) (parse-rhs <e2> ...)))
 
 		 ;; other internal $-syntax
+		 ((_ ($prec <tok>) <e2> ...)
+		  #'(cons (cons 'prec <tok>) (parse-rhs <e2> ...)))
 		 ((_ ($with <lhs-ref> <ex> ...) <e2> ...)
 		  #'(cons `(with <lhs-ref> ,@(with-attr-list <ex> ...))
 			  (parse-rhs <e2> ...)))
@@ -166,11 +170,13 @@
 	    (parse-precedence
 	     (syntax-rules (left right nonassoc)
 	       ((_ (left <tk> ...) <ex> ...)
-		(cons '(left <tk> ...) (parse-precedence <ex> ...)))
+		(cons (list 'left <tk> ...) (parse-precedence <ex> ...)))
 	       ((_ (right <tk> ...) <ex> ...)
-		(cons '(right <tk> ...) (parse-precedence <ex> ...)))
+		(cons (list 'right <tk> ...) (parse-precedence <ex> ...)))
 	       ((_ (nonassoc <tk> ...) <ex> ...)
-		(cons '(nonassoc <tk> ...) (parse-precedence <ex> ...)))
+		(cons (list 'nonassoc <tk> ...) (parse-precedence <ex> ...)))
+	       ((_ <tk> <ex> ...)
+		(cons (list 'undecl <tk>) (parse-precedence <ex> ...)))
 	       ((_) '())))
 	    (lalr-spec-1
 	     (syntax-rules (start expect notice prec< prec> grammar)
@@ -232,7 +238,7 @@
 (define (process-spec tree)
 
   ;; Make a new symbol. This is a helper for proxies and mid-rule-actions.
-  ;; The counter here is the one item in @code{process-spec} that is redefined.
+  ;; The counter here is the only @code{set!} in @code{process-spec}.
   ;; Otherwise, I believe @code{process-spec} is referentially transparent.
   (define maksy
     (let ((cntr 1))
@@ -249,7 +255,7 @@
     (let iter ((p-l '()) (a-l '()) (tree tree) (node '()) (pspec '()))
       (cond
        ((pair? pspec)
-	;; psp looks like ('left "+" "-"); pel ("+" "-"); psy (#\+ #\-)
+	;; psp ~ ('left "+" "-"); pel ~ ("+" "-"); psy ~ (#\+ #\-)
 	(let* ((psp (car pspec)) (pel (cdr psp)) (psy (map atomize pel)))
 	  (iter (append (x-comb psy node)
 			;; append (+ -) => ((+ -) (- +))
@@ -269,14 +275,16 @@
   (define (make-mra-proxy sy pel act)
     (list sy (list (cons* 'action (length pel) (cdr act)))))
 
-  ;; fatal: symbol used as terminal and non-terminal
+  ;; @item gram-check-2 tl nl err-l
+  ;; Check for fatal: symbol used as terminal and non-terminal.
   (define (gram-check-2 tl nl err-l)
     (let ((cf (lset-intersection eqv? (map atomize tl) nl)))
       (if (pair? cf)
 	  (cons (fmtstr "*** symbol is terminal and non-terminal: ~S" cf)
 		err-l) err-l)))
 	       
-  ;; fatal: non-terminal's w/o production rule		  
+  ;; @item gram-check-3 ll nl err-l
+  ;; Check for fatal: non-terminal's w/o production rule.
   (define (gram-check-3 ll nl err-l)
     (fold
      (lambda (n l)
@@ -285,7 +293,8 @@
 	   l))
      err-l nl))
 
-  ;; warning: unused LHS
+  ;; @item gram-check-4 ll nl err-l
+  ;; Check for warning: unused LHS.
   ;; TODO: which don't appear in OTHER RHS, e.g., (foo (foo))
   (define (gram-check-4 ll nl err-l)
     (fold
@@ -299,7 +308,9 @@
 		     ull (cons (car all) ull))
 		 (cdr all))))))
 	       
-  ;; warning: duplicate terminals under atomize (e.g., 'foo "foo")
+  ;; Check for warning: duplicate terminals under atomize (e.g., 'foo "foo").
+  ;; BUG: Get all of these warnings on other random errors.  This needs to be
+  ;; more robust.
   (define (gram-check-5 terminals prev-errs)
     (let ((f "warning: similar terminals: ~A ~A"))
       (let iter ((errs prev-errs) (head '()) (tail terminals))
@@ -311,98 +322,109 @@
 		    (cons pair head) (cdr tail)))))))
 
   (let* ((gram (assq-ref tree 'grammar))
-	 (start-symbol (assq-ref tree 'start))
+	 (start-symbol (and=> (assq-ref tree 'start) atomize))
 	 (start-rule (lambda () (list start-symbol)))
 	 (add-el (lambda (e l) (if (memq e l) l (cons e l))))
 	 (pna (prec-n-assc tree)))
-    ;; Sweep through the grammar to generate a canonical specification.
+    ;; We sweep through the grammar to generate a canonical specification.
     ;; Note: the local rhs is used to hold RHS terms, but a
     ;; value of @code{'()} is used to signal "add rule", and a value of
     ;; @code{#f} is used to signal ``done, proceed to next rule.''
     ;; We use @code{tail} below to go through all remaining rules so that any
     ;; like LHS get absorbed before proceeding: This keeps LHS in sequence.
     (let iter ((ll '($start))		; LHS list
-	       (rl (list (start-rule))) ; RHS list
-	       (al (list (list 1 'all '$1))) ; narg-ref-action list
-	       (xl (list 1))		     ; reduction size
+	       (@l (list		; attributes per prod' rule
+		    `((rhs . ,(vector start-symbol))
+		      (ref . all) (act 1 $1))))
 	       (tl (list '$end))	; set of terminals (add $end?)
 	       (nl (list start-symbol))	; set of non-terminals
-	       (zl (list))		; wierd stuff
 	       ;;
-	       (head gram)	       ; head of grammar productions
+	       (head gram)	       ; head of unprocessed productions
 	       (prox '())	       ; proxy productions for MRA
 	       (lhs #f)		       ; current LHS (symbol)
 	       (tail '())	       ; tail of grammar productions
 	       (rhs-l '())	       ; list of RHSs being processed
-	       (act #f)		       ; action if seen in RHS
-	       (pel '())	       ; processed (RHS) terms list
-	       (rhs #f))	       ; list of el'ts being processed
+	       (attr '())	       ; per-rule attributes (action, prec)
+	       (pel '())	       ; processed RHS terms: '$:if ...
+	       (rhs #f))	       ; elts to process: (terminal . '$:if) ...
       (cond
        ((pair? rhs)
 	;; Capture info on RHS term.
 	(case (caar rhs)
 	  ((terminal)
-	   (iter ll rl al xl (add-el (cdar rhs) tl) nl zl head prox lhs tail
-		 rhs-l act (cons (atomize (cdar rhs)) pel) (cdr rhs)))
+	   (iter ll @l (add-el (cdar rhs) tl) nl head prox lhs tail
+		 rhs-l attr (cons (atomize (cdar rhs)) pel) (cdr rhs)))
 	  ((non-terminal)
-	   (iter ll rl al xl tl (add-el (cdar rhs) nl) zl head prox lhs tail
-		 rhs-l act (cons (cdar rhs) pel) (cdr rhs)))
+	   (iter ll @l tl (add-el (cdar rhs) nl) head prox lhs tail
+		 rhs-l attr (cons (cdar rhs) pel) (cdr rhs)))
 	  ((action)
-	   (if (pair? (cdr rhs)) ;; not last elemnt in RHS
+	   (if (pair? (cdr rhs))
 	       ;; mid-rule action: generate a proxy (car act is # args)
 	       (let* ((sy (maksy))
 		      (pr (make-mra-proxy sy pel (cdar rhs))))
-		 (iter ll rl al xl tl (cons sy nl) zl head (cons pr prox)
-		       lhs tail rhs-l act (cons sy pel) (cdr rhs)))
+		 (iter ll @l tl (cons sy nl) head (cons pr prox)
+		       lhs tail rhs-l attr (cons sy pel) (cdr rhs)))
 	       ;; end-rule action
-	       (iter ll rl al xl tl nl zl head prox lhs tail
-		     rhs-l (cdar rhs) pel (cdr rhs))))
+	       (iter ll @l tl nl head prox lhs tail
+		     rhs-l (acons 'action (cdar rhs) attr) pel (cdr rhs))))
 	  ((proxy)
 	   (let* ((sy (maksy))
 		  (pf (cadar rhs))	; proxy function
 		  (p1 (pf sy (cddar rhs))))
-	     (iter ll rl al xl tl (cons sy nl) zl head (cons p1 prox) lhs tail
-		   rhs-l act (cons sy pel) (cdr rhs))))
-
+	     (iter ll @l tl (cons sy nl) head (cons p1 prox) lhs
+		   tail rhs-l attr (cons sy pel) (cdr rhs))))
+	  ((prec)
+	   (iter ll @l (add-el (cdar rhs) tl) nl head prox lhs tail rhs-l
+		 (acons 'prec (atomize (cdar rhs)) attr) pel (cdr rhs)))
 	  ((with)
-	   (let* ((psy (maksy))		; proxy symbol
-		  (rhsx (cadar rhs))	; symbol to expand
+	   (let* ((psy (maksy))		      ; proxy symbol
+		  (rhsx (cadar rhs))	      ; symbol to expand
 		  (p-l (map cdr (cddar rhs))) ; prune list
 		  (p1 (list psy `((non-terminal . ,rhsx)
 				  (action #f #f $1)))))
-	     (iter ll rl al xl tl (cons psy nl) (acons psy p-l zl) head
-		   (cons p1 prox) lhs tail rhs-l act (cons psy pel) (cdr rhs))))
-
+	     (iter ll @l tl (cons psy nl) head (cons p1 prox) lhs tail rhs-l
+		   (acons 'with (cons psy p-l) attr) (cons psy pel) (cdr rhs))))
 	  (else
 	   (error (fmtstr "bug=~S" (caar rhs))))))
 
        ((null? rhs)
 	;; End of RHS items for current rule.
+	;; Add the p-rules items to the lists ll, rl, xl, and @l.
+	;; @code{act} is now:
+	;; @itemize
+	;; @item for mid-rule-action: (narg ref code)
+	;; @item for end-rule-action: (#f ref code)
+	;; @end itemize
+	;;(simple-format #t "lhs=~S  ln=~A\n  act=~S\n" lhs (length pel) act)
 	(let* ((ln (length pel))
-	       (r1 (reverse pel))
-	       (a1 (cond ((and act (car act)) act) ; mid rule action
-			 (act (cons ln (cdr act))) ; standard act
-			 ((null? pel) (list 0 #f '(list))) ; def act w/ 0 RHS
-			 (else (list ln #f '$1))))) ; def act w/ 1+ RHS
-	  (iter (cons lhs ll) (cons r1 rl) (cons a1 al)
-		(cons ln xl) tl nl zl head prox lhs tail rhs-l act pel #f)))
+	       (action (assq-ref attr 'action))
+	       (nrg (if action (or (car action) ln) ln))  ; number of args
+	       (ref (if action (cadr action) #f))
+	       (act (if (and action (cddr action)) (cddr action)
+			(if (zero? nrg) '((list)) '($1)))))
+	  (iter (cons lhs ll)
+		(cons
+		 (cons* (cons 'rhs (list->vector (reverse pel)))
+			(cons* 'act nrg act) (cons 'ref ref) attr)
+		 @l)
+		tl nl head prox lhs tail rhs-l attr pel #f)))
 
        ((pair? rhs-l)
 	;; Work through next RHS.
-	(iter ll rl al xl tl nl zl head prox lhs tail
-	      (cdr rhs-l) #f '() (car rhs-l)))
+	(iter ll @l tl nl head prox lhs tail
+	      (cdr rhs-l) '() '() (car rhs-l)))
 
        ((pair? tail)
 	;; Check the next CAR of the tail.  If it matches
 	;; the current LHS process it, else skip it.
-	(iter ll rl al xl tl nl zl head prox lhs (cdr tail) 
+	(iter ll @l tl nl head prox lhs (cdr tail) 
 	      (if (eqv? (caar tail) lhs) (cdar tail) '())
-	      act pel #f))
+	      attr pel #f))
 
        ((pair? prox)
 	;; If a proxy then we have ((lhs RHS) (lhs RHS))
-	(iter ll rl al xl tl nl zl (cons (car prox) head) (cdr prox)
-		lhs tail rhs-l act pel rhs))
+	(iter ll @l tl nl (cons (car prox) head) (cdr prox)
+	      lhs tail rhs-l attr pel rhs))
 
        ((pair? head)
 	;; Check the next rule-set.  If the lhs has aready
@@ -411,15 +433,12 @@
 	(let ((lhs (caar head)) (rhs-l (cdar head))
 	      (rest (cdr head)))
 	  (if (memq lhs ll)
-	      (iter ll rl al xl tl nl zl rest prox
-		    #f '() '() act pel #f)
-	      (iter ll rl al xl tl nl zl rest prox
-		    lhs rest rhs-l act pel rhs))))
+	      (iter ll @l tl nl rest prox #f '() '() attr pel #f)
+	      (iter ll @l tl nl rest prox lhs rest rhs-l attr pel rhs))))
 
        (else
 	;;(simple-format #t "rl=~S\n" rl)
-	(let* ((rv (list->vector (map list->vector (reverse rl))))
-	       (ral (reverse al))
+	(let* ((al (reverse @l))	; attribute list
 	       (err-1 '()) ;; not used
 	       ;; symbol used as terminal and non-terminal
 	       (err-2 (gram-check-2 tl nl err-1))
@@ -438,9 +457,8 @@
 	       ;; most referenced
 	       (cons 'non-terms nl)
 	       (cons 'lhs-v (list->vector (reverse ll)))
-	       (cons 'rhs-v rv)
-	       (cons 'prune-al zl)	; new prunage al
-	       ;; not as much
+	       (cons 'rhs-v (map-attr->vector al 'rhs))
+	       ;;(cons 'prune-al TODO: filter 'with from '@l)
 	       (cons 'terminals tl)
 	       (cons 'start start-symbol)
 	       (cons 'attr (list (cons 'expect (or (assq-ref tree 'expect) 0))
@@ -448,9 +466,9 @@
 				 ))
 	       (cons 'prec (assq-ref pna 'prec))
 	       (cons 'assc (assq-ref pna 'assc))
-	       (cons 'act-v (list->vector (map cddr ral)))
-	       (cons 'ref-v (list->vector (map cadr ral)))
-	       (cons 'nrg-v (list->vector (map car ral)))
+	       (cons 'prp-v (map-attr->vector al 'prec)) ; per-rule precedence
+	       (cons 'act-v (map-attr->vector al 'act))
+	       (cons 'ref-v (map-attr->vector al 'ref))
 	       (cons 'err-l err-l)))))))))
   
 ;;; === Code for processing the specification. ================================
@@ -821,7 +839,7 @@
      (else
       rslt))))
 
-;; @item step1 [input-a-list] => output-a-list
+;; @item step1 [input-a-list] => p-mach-1
 ;; Compute the sets of LR(0) kernel items and the transitions associated with
 ;; spec.  These are returned as vectors in the alist with keys @code{'kis-v}
 ;; and @code{'kix-v}, repspectively.   Each entry in @code{kis-v} is a list of
@@ -1075,7 +1093,7 @@
      (else
       (set-cdr! ar (acons sx1 it1 (cdr ar))) #t))))
 
-;; @item step2 subm1 => subm2
+;; @item step2 p-mach-1 => p-mach-2
 ;; This implements steps 2 and 3 of Algorithm 4.13 on p. 242 of the DB.
 ;; The a-list @code{subm1} includes the kernel itemsets and transitions
 ;; from @code{step1}.   This routine adds two entries to the alist:
@@ -1090,9 +1108,9 @@
 ;;       otherwise add T to spontaneously-generated list
 ;; @end example
 ;;   
-(define (step2 subm)
-  (let* ((kis-v (assq-ref subm 'kis-v))
-	 (kix-v (assq-ref subm 'kix-v)) ; transitions?
+(define (step2 p-mach)
+  (let* ((kis-v (assq-ref p-mach 'kis-v))
+	 (kix-v (assq-ref p-mach 'kix-v)) ; transitions?
 	 (nkset (vector-length kis-v))	; number of k-item-sets
 	 ;; kernel-itemset tokens
 	 (kit-v (make-vector nkset '())) ; sx => alist: (item latoks)
@@ -1123,7 +1141,7 @@
 	      ;; End-items don't shift, so don't propagate.
 	      (remove last-item? (vector-ref kis-v (1+ kx)))))))
     (when #f (pp-kip-v kip-v) (pp-kit-v kit-v)) ; for debugging
-    (cons* (cons 'kit-v kit-v) (cons 'kip-v kip-v) subm)))
+    (cons* (cons 'kit-v kit-v) (cons 'kip-v kip-v) p-mach)))
 
 ;; debug for step2
 (define (pp-kit ix kset)
@@ -1146,11 +1164,12 @@
   (fmtout "propagate:\n")
   (vector-for-each pp-kip kip-v))
 
-;; @item step3 subm2 => subm3
+;; @item step3 p-mach-2 => p-mach-3
+;; Execute nyacc step 3, where p-mach means ``partial machine''.
 ;; This implements step 4 of Algorithm 4.13 from the DB.
-(define (step3 subm)
-  (let* ((kit-v (assq-ref subm 'kit-v))
-	 (kip-v (assq-ref subm 'kip-v))
+(define (step3 p-mach)
+  (let* ((kit-v (assq-ref p-mach 'kit-v))
+	 (kip-v (assq-ref p-mach 'kip-v))
 	 (nkset (vector-length kit-v)))
     (let iter ((upd #t)			; token propagated?
 	       (kx -1)			; current index
@@ -1177,7 +1196,7 @@
        (upd
 	;; Have updates, rerun.
 	(iter #f 0 '() '() '() '()))))
-    subm))
+    p-mach))
 
 ;; @item reductions kit-v sx => ((tokA gxA1 ...) (tokB gxB1 ...) ...)
 ;; This is a helper for @code{step4}.
@@ -1323,9 +1342,8 @@
      (else res))))
 
 
-
 ;; This is a helper for step4.
-(define (prev-symb act its)
+(define (prev-sym act its)
   (let* ((a act)
 	 (tok (car a)) (sft (caddr a)) (red (cdddr a))
 	 ;; @code{pit} is the end-item in the p-rule to be reduced.
@@ -1334,7 +1352,7 @@
 	 (psy (looking-at pit)))
     psy))
 
-;; @item step4 subm => more-subm
+;; @item step4 p-mach-0 => p-mach-1
 ;; This generates the parse action table from the itemsets and then applies
 ;; precedence and associativity rules to eliminate shift-reduce conflicts
 ;; where possible.  The output includes the parse action table (entry
@@ -1346,17 +1364,19 @@
 ;; if reduce by zero we are done. so never hit state zero accept on ACCEPT?
 ;; For each state, the element of pat-v looks like
 ;; ((tokenA . (reduce . 79)) (tokenB . (reduce . 91)) ... )
-(define (step4 subm)
-  (let* ((kis-v (assq-ref subm 'kis-v)) ; states
-	 (kit-v (assq-ref subm 'kit-v)) ; la-toks
-	 (kix-v (assq-ref subm 'kix-v)) ; transitions
-	 (assc (assq-ref subm 'assc))	 ; associativity rules
-	 (prec (assq-ref subm 'prec))	 ; precedence rules
+(define (step4 p-mach)
+  (let* ((kis-v (assq-ref p-mach 'kis-v)) ; states
+	 (kit-v (assq-ref p-mach 'kit-v)) ; la-toks
+	 (kix-v (assq-ref p-mach 'kix-v)) ; transitions
+	 (assc (assq-ref p-mach 'assc))	 ; associativity rules
+	 (prec (assq-ref p-mach 'prec))	 ; precedence rules
 	 (nst (vector-length kis-v))	 ; number of states
 	 (pat-v (make-vector nst '()))	 ; parse-action table per state
 	 (rat-v (make-vector nst '()))	 ; removed-action table per state
 	 (gen-pat-ix (lambda (ix)	 ; pat from shifts and reduc's
 		       (gen-pat (vector-ref kix-v ix) (reductions kit-v ix))))
+	 (prp-v (assq-ref p-mach 'prp-v))  ; per-rule precedence
+	 (tl (assq-ref p-mach 'terminals)) ; for error msgs
 	 )
     ;; We run through each itemset.
     ;; @enumerate
@@ -1380,15 +1400,21 @@
 	  ((srconf)
 	   (let* ((act (car actl))
 		  (tok (car act)) (sft (caddr act)) (red (cdddr act))
-		  (psy (prev-symb act (vector-ref kis-v ix)))
-		  (pre (prece psy tok prec))
+		  (prp (vector-ref prp-v red))
+		  (psy (prev-sym act (vector-ref kis-v ix)))
+		  (preced (or (and prp (prece prp tok prec)) ; rule-based
+			      (prece psy tok prec))) ; oper-based
 		  (sft-a (cons* tok 'shift sft))
 		  (red-a (cons* tok 'reduce red)))
 	     (call-with-values
 		 (lambda ()
-		   ;; If precedence applies use that else use associativity.
-		   (case pre ;; precedence
-		     ((#\=) ;; use associativity
+		   ;; Use precedence or, if =, associativity.
+		   (case preced
+		     ((#\>)
+		      (values red-a (cons sft-a 'pre) #f #f))
+		     ((#\<)
+		      (values sft-a (cons red-a 'pre) #f #f))
+		     ((#\=) ;; Now use associativity
 		      (case (assq-ref assc tok)
 			((left)
 			 (values red-a (cons sft-a 'ass) #f #f))
@@ -1398,12 +1424,8 @@
 			 (values (cons* tok 'error red) #f #f (cons ix act)))
 			(else
 			 (values sft-a (cons red-a 'def) (cons ix act) #f))))
-		     ((#\>)
-		      (values red-a (cons sft-a 'pre) #f #f))
-		     ((#\<)
-		      (values sft-a (cons red-a 'pre) #f #f))
-		     (else
-		      (values sft-a (cons red-a 'pre) (cons ix act) #f))))
+		     (else ;; Or default, which is shift.
+		      (values sft-a (cons red-a 'def) (cons ix act) #f))))
 	       (lambda (a r w f)
 		 (iter ix
 		       (if a (cons a pat) pat)
@@ -1412,8 +1434,8 @@
 		       (if f (cons f ftl) ftl)
 		       (cdr actl))))))
 	  ((rrconf)
-	   (fmterr "*** reduce-reduce conflict: in state ~A on ~A: ~A\n"
-		   ix (caar actl) (cddar actl))
+	   #;(fmterr "*** reduce-reduce conflict: in state ~A on ~A: ~A\n"
+		   ix (obj->str (find-terminal (caar actl) tl)) (cddar actl))
 	   (iter ix (cons (car actl) pat) rat wrn
 		 (cons (cons ix (car actl)) ftl) (cdr actl)))
 	  (else
@@ -1425,16 +1447,17 @@
        ((< (1+ ix) nst)
 	(iter (1+ ix) '() '() wrn ftl (gen-pat-ix (1+ ix))))
        (else
-	(let* ((attr (assq-ref subm 'attr))
+	(let* ((attr (assq-ref p-mach 'attr))
 	       (expect (assq-ref attr 'expect))) ; expected # srconf
 	  (if (not (= (length wrn) expect))
 	      (for-each (lambda (m) (fmterr "+++ warning: ~A\n" (conf->str m)))
 			(reverse wrn)))
-	  (for-each (lambda (m) (fmterr "*** fatal  : ~S\n" m))
-		    (reverse ftl))
-	))))
+	  (for-each
+	   (lambda (m) (fmterr "*** fatal: ~A\n" (conf->str m)))
+	   (reverse ftl))
+	  ))))
     ;; Return mach with parse-action and removed-action tables.
-    (cons* (cons 'pat-v pat-v) (cons 'rat-v rat-v) subm)))
+    (cons* (cons 'pat-v pat-v) (cons 'rat-v rat-v) p-mach)))
 
 ;; @item conf->str cfl => string
 ;; map conflict (e.g., @code{('rrconf 1 . 2}) to string.
@@ -1450,66 +1473,33 @@
 	    (obj->str (find-terminal tok terms)))))
 		     
 ;; @item gen-match-table mach => mach
-;; Generate the match-table for a machine.  The match-table may be passed to
+;; Generate the match-table for a machine.  The match table is a list of
+;; pairs: the car is the token used in the grammar specification, the cdr
+;; is the symbol that should be returned by the lexical analyzer.
+;;
+;; The match-table may be passed to
 ;; the lexical analyzer builder to identify strings or string-types as tokens.
-;; Key is @code{mtab}.  The format is as follows
+;; The associated key in the machine is @code{mtab}. 
 ;; @enumerate
 ;; @item
 ;; @sc{nyacc}-reserved symbols are provided as symbols
 ;; @example
-;; $symbol -> ($symbol . $symbol)
+;; $ident -> ($ident . $ident)
 ;; @end example
 ;; @item
 ;; Terminals used as symbols (@code{'comment} versus @code{"comment"}) are
-;; provided as symbols.  The spec parser should warn if symbols are used in
-;; both ways.  (This is a conflict.)
+;; provided as symbols.  The spec parser will provide a warning if symbols
+;; are used in both ways.
 ;; @item
 ;; Others are provided as strings.
 ;; @end enumerate
 ;; The procedure @code{hashify-machine} will convert the cdrs to integers.
 ;; Test: "$abc" => ("$abc" '$abc) '$abc => ('$abc . '$abc)
-;; FIX: need to return to old-gen-match-table and make lexer sync up with that.
-(define (old1-gen-match-table mach)
-  (let ((terminals (assq-ref mach 'terminals)))
-    (cons*
-     (cons 'mtab
-	   (let iter ((mt '(($default . $default) ($error . $error)))
-		      (tl terminals))
-	     (if (null? tl) (reverse mt)
-		 (let* ((tok (car tl))
-			(str (if (symbol? tok) (symbol->string tok) #f))
-			(rez (and str (eqv? #\$ (string-ref str 0)))))
-		   (iter (acons (cond (rez tok) (str str) (else tok))
-				(atomize tok)
-				mt)
-			 (cdr tl))))))
-     mach)))
-(define (old2-gen-match-table mach)
-  (let ((terminals (assq-ref mach 'terminals)))
-    (cons*
-     (cons 'mtab
-	   (let iter ((mt '(($default . $default) ($error . $error)))
-		      (tl terminals))
-	     (if (null? tl) (reverse mt)
-		 (let* ((tok (car tl))
-			(str (cond
-			      ((symbol? tok) (symbol->string tok))
-			      ((char? tok) (list->string (list tok)))
-			      (else tok)))
-			(rez (and (symbol? tok) (eqv? #\$ (string-ref str 0)))))
-		   (iter (acons (if rez tok str) (atomize tok) mt)(cdr tl))))))
-     mach)))
 (define (gen-match-table mach)
   (cons
    (cons 'mtab (map (lambda (term) (cons term (atomize term)))
 		    (assq-ref mach 'terminals)))
    mach))
-(define (old4-gen-match-table mach)
-  (acons
-   'mtab (fold (lambda (term seed) (cons (cons term (atomize term)) seed))
-	       '(($default . $default))
-	       (assq-ref mach 'terminals))
-   mach))
 
 ;; @item lalr-match-table mach => match-table
 ;; Get the match-table
@@ -1523,7 +1513,8 @@
 	 (nst (vector-length pat-v))
 	 (hashed (number? (caar (vector-ref pat-v 0)))) ; been hashified?
 	 (reduce? (if hashed
-		      (lambda (a) (negative? a))
+		      (lambda (a) (and (number? a) ; check number: #f for xrconf
+				       (negative? a)))
 		      (lambda (a) (eq? 'reduce (car a)))))
 	 (reduce-pr (if hashed abs cdr))
 	 (reduce-to? (if hashed
@@ -1585,7 +1576,6 @@
 ;; to build parser, need:
 ;;   pat-v - parse action table
 ;;   ref-v - references
-;;   nrg-v - number of args
 ;;   len-v - rule lengths
 ;;   rto-v - rule lengths
 ;; to print itemsets need:
@@ -1657,12 +1647,14 @@
 
 ;; @item pp-lalr-grammar spec [port]
 ;; Pretty-print the grammar to the specified port, or current output.
+;; TODO: Add @code{$prec} and @code{$prune} expressions.
 (define (pp-lalr-grammar spec . rest)
   (let* ((port (if (pair? rest) (car rest) (current-output-port)))
 	 (lhs-v (assq-ref spec 'lhs-v))
 	 (rhs-v (assq-ref spec 'rhs-v))
 	 (nrule (vector-length lhs-v))
 	 (act-v (assq-ref spec 'act-v))
+	 ;;(prp-v (assq-ref mach 'prp-v)) ; per-rule precedence
 	 (terms (assq-ref spec 'terminals))
 	 (prev-core (fluid-ref *lalr-core*)))
     (fluid-set! *lalr-core* (make-core spec)) ; OR dynamic-wind ???
@@ -1688,7 +1680,6 @@
 	 (lhs-v (assq-ref mach 'lhs-v))
 	 (rhs-v (assq-ref mach 'rhs-v))
 	 (nrule (vector-length lhs-v))
-	 (act-v (assq-ref mach 'act-v))
 	 (pat-v (assq-ref mach 'pat-v))
 	 (rat-v (assq-ref mach 'rat-v))
 	 (kis-v (assq-ref mach 'kis-v))
@@ -1722,10 +1713,11 @@
 	       (let ((sy (car act)) (pa (cadr act)) (gt (cddr act)))
 		 (case pa
 		   ((srconf)
-		    (fmt port "\t\t~S => CONFLICT: shift ~A, reduce ~A\n" sy
-			 (car gt) (cdr gt)))
+		    (fmt port "\t\t~A => CONFLICT: shift ~A, reduce ~A\n"
+			 (elt->str sy terms) (car gt) (cdr gt)))
 		   ((rrconf)
-		    (fmt port "\t\t~S => CONFLICT: reduce ~A\n" sy
+		    (fmt port "\t\t~A => CONFLICT: reduce ~A\n"
+			 (elt->str sy terms)
 			 (string-join (map number->string gt) ", reduce ")))
 		   (else
 		    (fmt port "\t\t~A => ~A ~A\n" (elt->str sy terms) pa gt))))
@@ -1766,8 +1758,8 @@
 ;; @enumerate
 ;; @item the @code{$end} marker
 ;; @item identifiers (using the symbolic token @code{$ident}
-;; @item non-negative integers (using the symbolic token @code{$fx})
-;; @item non-negative floats (using the symbolic token @code{$fl})
+;; @item non-negative integers (using the symbolic token @code{$fixed})
+;; @item non-negative floats (using the symbolic token @code{$float})
 ;; @item @code{$default} => 0
 ;; @end enumerate
 ;; And action
@@ -1794,9 +1786,14 @@
 ;; this should be done as part of @code{make-lexer}, by filtering the token
 ;; list through the ident-reader.
 ;; NOTE: The parser is hardcoded to assume that the phony token for the
-;; default action is @code{'$default} for unhashed machine or @code{-1} for
-;; a hashed machine.
+;; default (reduce) action is @code{'$default} for unhashed machine or
+;; @code{-1} for a hashed machine.
+
+;; NEW: need to add reduction of ERROR
+
 ;; @item hashify-machine mach => mach
+;; There is a bug in here: if pat contains rrconf then the output is
+;; #unspedified.
 (define (hashify-machine mach)
   (let* ((terminals (assq-ref mach 'terminals))
 	 (non-terms (assq-ref mach 'non-terms))
@@ -1828,7 +1825,8 @@
 		     (tk (car a0)) (ac (cadr a0)) (ds (cddr a0))
 		     ;; t: encoded token; d: encoded destination
 		     (t (sym->int tk))
-		     (d (case ac ((shift) ds) ((reduce) (- ds)) ((accept) 0))))
+		     (d (case ac ((shift) ds) ((reduce) (- ds)) ((accept) 0)
+			      (else #f))))
 		(unless t
 		  (fmterr "~S ~S ~S\n" tk ac ds) 
 		  (error "expect something"))
@@ -1852,138 +1850,8 @@
 (define (machine-hashed? mach)
   (number? (caar (vector-ref (assq-ref mach 'pat-v) 0))))
 
-;; @item make-arg-list N => '($N $Nm1 $Nm2 ... $1 . $rest)
-;; This is a helper for @code{mkact}.
-(define (make-arg-list n)
-  (let ((mkarg
-	 (lambda (i) (string->symbol (string-append "$" (number->string i))))))
-    (let iter ((r '(. $rest)) (i 1))
-      (if (> i n) r (iter (cons (mkarg i) r) (1+ i))))))
-
-;; @item wrap-action n act => `(lambda ($n ... $2 $1 . $rest) ,@act)
-;; Wrap user-specified action (body, as list) of n arguments in a lambda.
-;; The rationale for the arglist format is that we can @code{apply} this
-;; lambda to the the semantic stack.
-(define (wrap-action n act)
-  (cons* 'lambda (make-arg-list n) act))
-
-;; @item make-lalr-parser mach => parser
-;; This generates a procedure that takes one argument, a lexical analyzer:
-;; @example
-;; (parser lexical-analyzer [#:debug #t])
-;; @end example
-;; and is used as
-;; @example
-;; (define xyz-parse (make-lalr-parser xyz-mach))
-;; (with-input-from-file "sourcefile.xyz" (lambda () (xyz-parse (gen-lexer))))
-;; @end example
-;; The generated parser is reentrant.
-(define* (make-lalr-parser mach)
-  (let* ((len-v (assq-ref mach 'len-v))
-	 (rto-v (assq-ref mach 'rto-v))	; reduce to
-	 (pat-v (assq-ref mach 'pat-v))
-	 (act-v (cond
-		 ((assq-ref mach 'nrg-v) =>
-		  (lambda (v)
-		    (vector-map
-		     ;; Turn symbolic action into executable procedures:
-		     (lambda (ix f) (eval f (current-module)))
-		     (vector-map
-		      ;; symbolic action
-		      (lambda (ix nrg guts) (wrap-action nrg guts))
-		      (assq-ref mach 'nrg-v) (assq-ref mach 'act-v)))))
-		 (else ;; have tables: no conversion needed
-		  (assq-ref mach 'act-v))))
-	 ;;
-	 (dmsg (lambda (s t a) (fmtout "state ~S, token ~S\t=> ~S\n" s t a)))
-	 (hashed (number? (caar (vector-ref pat-v 0)))) ; been hashified?
-	 ;;(def (assq-ref (assq-ref mach 'mtab) '$default))
-	 (def (if hashed -1 '$default))
-	 ;; predicate to test for shift action:
-	 (shift? (if hashed
-		     (lambda (a) (positive? a))
-		     (lambda (a) (eq? 'shift (car a)))))
-	 ;; On shift, transition to this state:
-	 (shift-to (if hashed (lambda (x) x) (lambda (x) (cdr x))))
-	 ;; predicate to test for reduce action:
-	 (reduce? (if hashed
-		      (lambda (a) (negative? a))
-		      (lambda (a) (eq? 'reduce (car a)))))
-	 ;; On reduce, reduce this production-rule:
-	 ;;(reduce-pr (if hashed (lambda (a) (abs a)) (lambda (a) (cdr a))))
-	 (reduce-pr (if hashed abs cdr))
-	 ;; If no action found in transition list, then this:
-	 (parse-error (if hashed #f (cons 'error 0)))
-	 ;; predicate to test for error
-	 (error? (if hashed
-		     (lambda (a) (eq? #f a))
-		     (lambda (a) (eq? 'error (car a)))))
-	 )
-    (lambda* (lexr #:key debug)
-      (let iter ((state (list 0))	; state stack
-		 (stack (list '$@))	; sval stack
-		 (nval #f)		; prev reduce to non-term val
-		 (lval (lexr)))		; lexical value (from lex'er)
-	(let* ((tval (car (if nval nval lval))) ; token (syntax value)
-	       (sval (cdr (if nval nval lval))) ; semantic value
-	       (stxl (vector-ref pat-v (car state))) ; state transition list
-	       (stx (or (assq-ref stxl tval) ; trans action (e.g. shift 32)
-			(assq-ref stxl def)  ; default action
-			parse-error)))
-	  (when debug (dmsg (car state) (if nval tval sval) stx))
-	  (cond
-	   ((error? stx)
-	    ;; Ugly to have to check this first every time, but
-	    ;; @code{positive?} and @code{negative?} fail otherwise.
-	    (let ((fn (or (port-filename (current-input-port)) "(unknown)"))
-		  (ln (1+ (port-line (current-input-port)))))
-	      (fmterr "~A: ~A: parse failed at state ~A, on input ~S\n"
-		      fn ln (car state) sval))
-	    #f)
-	   ((shift? stx)
-	    ;; We could check here to determine if next transition only has a
-	    ;; default reduction and, if so, go ahead and process the reduction
-	    ;; without reading another input token.  Needed for interactive.
-	    (iter (cons (shift-to stx) state) (cons sval stack)
-		  #f (if nval lval (lexr))))
-	   ((reduce? stx)
-	    (let* ((gx (reduce-pr stx)) (gl (vector-ref len-v gx))
-		   ($$ (apply (vector-ref act-v gx) stack)))
-	      (iter (list-tail state gl) 
-		    (list-tail stack gl)
-		    (cons (vector-ref rto-v gx) $$)
-		    lval)))
-	   (else ;; accept
-	    (car stack))))))))
-
-#;(define hashed-parser-code
-  '(define* (lexr #:key debug)
-     (define (dmsg s t a) (fmtout "state ~S, token ~S\t=> ~S\n" s t a))
-     (let iter ((state (list 0)) (stack (list '$@)) (nval #f) (lval (lexr)))
-       (let* ((tval (car (if nval nval lval)))
-	      (sval (cdr (if nval nval lval)))
-	      (stxl (vector-ref pat-v (car state)))
-	      (stx (or (assq-ref stxl tval) (assq-ref stxl -1) #f)))
-	 (when debug (dmsg (car state) (if nval tval sval) stx))
-	 (cond
-	  ((eq? #f stx)
-	   (let ((fn (or (port-filename (current-input-port)) "(unknown)"))
-		 (ln (1+ (port-line (current-input-port)))))
-	     (fmterr "~A: ~A: parse failed at state ~A, on input ~S\n"
-		     fn ln (car state) sval))
-	   #f)
-	  ((positive? stx)
-	   (iter (cons (shift-to stx) state) (cons sval stack)
-		 #f (if nval lval (lexr))))
-	  ((negative? stx)
-	   (let* ((gx (abs stx)) (gl (vector-ref len-v gx))
-		  ($$ (apply (vector-ref act-v gx) stack)))
-	     (iter (list-tail state gl) 
-		   (list-tail stack gl)
-		   (cons (vector-ref rto-v gx) $$)
-		   lval)))
-	  (else (car stack)))))))
 
+;; === output routines ===============
 
 (use-modules (ice-9 pretty-print))
 (use-modules (ice-9 regex))
@@ -2037,10 +1905,10 @@
      (lambda ()
        (fmt port "(define act-v\n  (vector\n")
        (vector-for-each
-	(lambda (ix nrg act)
+	(lambda (ix actn)
 	  (fmt port "   ;; ~A\n" (pp-rule/ts ix))
-	  (pretty-print (wrap-action nrg act) port #:per-line-prefix "   "))
-	(assq-ref mach 'nrg-v) (assq-ref mach 'act-v))
+	  (pretty-print (wrap-action actn) port #:per-line-prefix "   "))
+	(assq-ref mach 'act-v))
        (fmt port "   ))\n\n"))))
 
   (call-with-output-file filename
@@ -2078,10 +1946,10 @@
      (lambda ()
        (fmt port "(define act-v\n  (vector\n")
        (vector-for-each
-	(lambda (ix nrg act)
+	(lambda (ix actn)
 	  (fmt port "   ;; ~A\n" (pp-rule/ts ix))
-	  (pretty-print (wrap-action nrg act) port #:per-line-prefix "   "))
-	(assq-ref mach 'nrg-v) (assq-ref mach 'act-v))
+	  (pretty-print (wrap-action actn) port #:per-line-prefix "   "))
+	(assq-ref mach 'act-v))
        (fmt port "   ))\n\n"))))
 
   (call-with-output-file filename
@@ -2102,4 +1970,4 @@
 
 ;; @end itemize
 
-;;; --- last line
+;;; --- last line ---
